# Upgrading from older versions

## Upgrading from v2.x to later versions

If you happen to be working on a website with v2.x installed and want to upgrade,
plan ahead for a solid amount of work to do to fit with the new standards of the
later versions.

The first thing to note is that there are many differences in v3.x onwards. One of the
main things is that nested preprocessing of entities in entity reference fields
no longer happens automatically. This is because v2.x had a lot of risks with this
behaviour. Infinite loops were much more prevalent.

With this major change, you will have to refactor most of your existing code
pertaining to harmonize.

When `entity_reference` fields are processed, a **Harmonizer** is returned
instead of the expected data of the referenced entities. To reach the nested
data, you simply have to call `->harmonize()` on the Harmonizer object. Keep
this in mind when refactoring your code.

### Upgrade Steps

Follow these steps to assure the least amount of headache when upgrading.

* Assure site configurations in your codebase are up-to-date: `drush cex -y`.
* Update the module via composer: `composer update drupal/harmonize`.
* Run database updates: `drush updb -y`.
* **Important** - Export migrated configurations.
* Now you should be good to go...To start refactoring!

### ^3+ Changelog

Below are other things to consider when upgrading from v2.x.

* Nested processing of referenced entities is no longer automatic. Adjust code accordingly.
* Harmonizer Signatures have been fine-tuned. When targeting entities in a specific style, you must adjust your harmony file names and Refiner targets.
  * `entity.node.article--card` becomes `entity.node.article.card`.
* Data configurations have been deprecated. They have been replaced with [Entity Processing Rules](#entity-processing-rules).
* Refiners should be good as they are thankfully.
* Theme Harmony configuration YAML files have been adjusted.
  * `THEMENAME.harmony.yml` is no longer loaded. The module will now load two separate files.
  * `THEMENAME.harmony.styles.yml` for Styles.
  * `THEMENAME.harmony.rules.yml` for Entity Rules.
  * See the `harmonize_examples` module for examples of the file structures. Examples will also be available in this guide soon.
* Theme `.harmony.php` files have been adjusted. Harmonize will now look for `.harmony` files instead.
  * You will have to change all `.harmony.php` file extensions to simply `.harmony`.
* An `$object` variable is now available in `.harmony` files. This variable contains the original object being preprocessed.
* [Styles](#styles) can now be created in the admin interface.
* A [Visualizer](#visualizer) has been added to allow entity processing tests.
* New [Twig](#twig-functions--filters) functionality has been added.
  * This may help with refactoring in Twig templates.
* [Caching](#caching) has been optimized and is no longer experimental. Enable it. Trust me.
* [Entity Settings](#entity-settings) have been enabled along with view mode integration.
  * Coming from V2, this may not be needed. But it may be something to consider during refactoring.
* [Main Module Settings](#modules-main-settings) have been adjusted.
  * Moved Automatic Preprocessing setting to a textarea. You now specify which entity types are automatically processed in entity reference fields.
    * **WARNING**: This setting can lead to infinite loops in some cases. Be mindful of the entity types you add here.
  * Removed recursive depth settings. No longer needed. We trust the devs to be mindful of configurations!
  * (3.x) - Added new setting for data alteration object loading. See [this section](#data-alteration) for more details.
    * If you're coming from v2.x and you have a lot of data alteration objects, you may want to set this to [**SCAN**](#scan-elevator).
    * If upgrading to v4.x, this setting was removed. **FCFS** is the preferred way moving forward.
