<?php

/**
 * @file
 * This is simply a file to declare reusable functions in your .harmony files.
 */

/**
 * Detect if a video is either from YouTube or Vimeo.
 *
 * @param string $url
 *   Absolute URL leading to the video itself.
 *
 * @return string|null
 *   The name of the provider found.
 */
function extract_remote_video_provider_from_url(string $url): ?string {
  $youtubeRegex = '/^(https?:\/\/)?(www\.youtube\.com|youtu\.?be)\/.+$/';
  $vimeoRegex = '/vimeo\.com\/\d+/';

  if (preg_match($youtubeRegex, $url)) {
    return 'youtube';
  }

  if (preg_match($vimeoRegex, $url)) {
    return 'vimeo';
  }

  return 'unknown';
}

/**
 * Extract a YouTube or Vimeo ID from their URL.
 *
 * @param string $provider
 *   Either YouTube or Vimeo.
 * @param string $url
 *   Absolute URL leading to the video itself.
 *
 * @return string|null
 *   The video unique identifier.
 */
function extract_remote_video_id_from_url(string $provider, string $url): ?string {
  switch ($provider) {
    case 'youtube':
      $regex
        = '/^.*(?:youtu\.be\/|v\/|u\/\w\/|embed\/|watch\?v=|&v=)([^#&?]*).*$/i';
      break;

    case 'vimeo':
      $regex
        = '/(?:www\.|player\.)?vimeo.com\/(?:channels\/(?:\w+\/)?|groups\/[^\/]*\/videos\/|album\/\d+\/video\/|video\/|)(\d+)(?:[a-zA-Z\d_\-]+)?/i';
      break;

    default:
      return NULL;
  }

  preg_match($regex, $url, $matches);
  return $matches[1] ?? NULL;
}
